#ifndef SRC_ADAPTIVETIMESTEPPER_HH
#define SRC_ADAPTIVETIMESTEPPER_HH

#include <fstream>

#include "coupledtimestepper.hh"

template <class Factory, class Updaters, class ErrorNorm>
class AdaptiveTimeStepper {
  using Vector = typename Factory::Vector;
  using ConvexProblem = typename Factory::ConvexProblem;
  using Nonlinearity = typename ConvexProblem::NonlinearityType;

  using MyCoupledTimeStepper = CoupledTimeStepper<Factory, Updaters, ErrorNorm>;

public:
  AdaptiveTimeStepper(Factory &factory, Dune::ParameterTree const &parset,
                      std::shared_ptr<Nonlinearity> globalFriction,
                      Updaters &current, double relativeTime,
                      double relativeTau,
                      std::function<void(double, Vector &)> externalForces,
                      ErrorNorm const &errorNorm,
                      std::function<bool(Updaters &, Updaters &)> mustRefine);

  bool reachedEnd();
  bool coarsen();
  void refine();
  void advance();

  double relativeTime_;
  double relativeTau_;

private:
  void stepAndReport(std::string type, MyCoupledTimeStepper &stepper,
                     double rTime, double rTau);

  double finalTime_;
  Factory &factory_;
  Dune::ParameterTree const &parset_;
  std::shared_ptr<Nonlinearity> globalFriction_;
  Updaters &current_;
  Updaters R1_;
  Updaters R2_;
  std::function<void(double, Vector &)> externalForces_;
  std::function<bool(Updaters &, Updaters &)> mustRefine_;
  std::fstream iterationWriter_;
  ErrorNorm const &errorNorm_;
};
#endif
